@@ -23,30 +23,6 @@ SMPTE2084_C3 = (2392.0 / 4096) * 32
 SRGB_K0 = 0.04045  # 0.055 / (2.4 - 1)
 SRGB_P = 12.92  # get_transfer_function_phi(0.055, 2.4)
 
-standard_illuminants = {
-	# 1st level is the standard name => illuminant definitions
-	# 2nd level is the illuminant name => CIE XYZ coordinates
-	# (Y should always assumed to be 1.0 and is not explicitly defined)
-	None: {"E": {"X": 1.00000, "Z": 1.00000}},
-	"ASTM E308-01": {"A": {"X": 1.09850, "Z": 0.35585},
-					 "C": {"X": 0.98074, "Z": 1.18232},
-					 "D50": {"X": 0.96422, "Z": 0.82521},
-					 "D55": {"X": 0.95682, "Z": 0.92149},
-					 "D65": {"X": 0.95047, "Z": 1.08883},
-					 "D75": {"X": 0.94972, "Z": 1.22638},
-					 "F2": {"X": 0.99186, "Z": 0.67393},
-					 "F7": {"X": 0.95041, "Z": 1.08747},
-					 "F11": {"X": 1.00962, "Z": 0.64350}},
-	"ICC": {"D50": {"X": 0.9642, "Z": 0.8249},
-			"D65": {"X": 0.9505, "Z": 1.0890}},
-	"Wyszecki & Stiles": {"A": {"X": 1.09828, "Z": 0.35547},
-						  "B": {"X": 0.99072, "Z": 0.85223},
-						  "C": {"X": 0.98041, "Z": 1.18103},
-						  "D55": {"X": 0.95642, "Z": 0.92085},
-						  "D65": {"X": 0.95017, "Z": 1.08813},
-						  "D75": {"X": 0.94939, "Z": 1.22558}}
-}
-
 
 def specialpow(a, b, slope_limit=0):
 	"""
@@ -179,7 +155,7 @@ rgb_spaces = {
 	#
 	# name              gamma             white                     primaries
 	#                                     point                     Rx      Ry      RY          Gx      Gy      GY          Bx      By      BY
-	"ACES RGB":         (1.0,             (0.95265, 1.0, 1.00883), (0.7347, 0.2653, 0.343961), (0.0000, 1.0000, 0.728164), (0.0001,-0.0770,-0.072125)),
+	"ACES":             (1.0,             (0.95265, 1.0, 1.00883), (0.7347, 0.2653, 0.343961), (0.0000, 1.0000, 0.728164), (0.0001,-0.0770,-0.072125)),
 	"Adobe RGB (1998)": (2 + 51 / 256.0,  "D65",                   (0.6400, 0.3300, 0.297361), (0.2100, 0.7100, 0.627355), (0.1500, 0.0600, 0.075285)),
 	"Apple RGB":        (1.8,             "D65",                   (0.6250, 0.3400, 0.244634), (0.2800, 0.5950, 0.672034), (0.1550, 0.0700, 0.083332)),
 	"Best RGB":         (2.2,             "D50",                   (0.7347, 0.2653, 0.228457), (0.2150, 0.7750, 0.737352), (0.1300, 0.0350, 0.034191)),
@@ -187,18 +163,19 @@ rgb_spaces = {
 	"Bruce RGB":        (2.2,             "D65",                   (0.6400, 0.3300, 0.240995), (0.2800, 0.6500, 0.683554), (0.1500, 0.0600, 0.075452)),
 	"CIE RGB":          (2.2,             "E",                     (0.7350, 0.2650, 0.176204), (0.2740, 0.7170, 0.812985), (0.1670, 0.0090, 0.010811)),
 	"ColorMatch RGB":   (1.8,             "D50",                   (0.6300, 0.3400, 0.274884), (0.2950, 0.6050, 0.658132), (0.1500, 0.0750, 0.066985)),
-	"DCI P3 RGB":       (2.6,             (0.89459, 1.0, 0.95442), (0.6800, 0.3200, 0.209475), (0.2650, 0.6900, 0.721592), (0.1500, 0.0600, 0.068903)),
+	"DCI P3":           (2.6,             (0.89459, 1.0, 0.95442), (0.6800, 0.3200, 0.209475), (0.2650, 0.6900, 0.721592), (0.1500, 0.0600, 0.068903)),
 	"Don RGB 4":        (2.2,             "D50",                   (0.6960, 0.3000, 0.278350), (0.2150, 0.7650, 0.687970), (0.1300, 0.0350, 0.033680)),
 	"ECI RGB":          (1.8,             "D50",                   (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
 	"ECI RGB v2":       (-3.0,            "D50",                   (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
 	"Ekta Space PS5":   (2.2,             "D50",                   (0.6950, 0.3050, 0.260629), (0.2600, 0.7000, 0.734946), (0.1100, 0.0050, 0.004425)),
-	"NTSC 1953 RGB":    (2.2,             "C",                     (0.6700, 0.3300, 0.298839), (0.2100, 0.7100, 0.586811), (0.1400, 0.0800, 0.114350)),
-	"PAL/SECAM RGB":    (2.2,             "D65",                   (0.6400, 0.3300, 0.222021), (0.2900, 0.6000, 0.706645), (0.1500, 0.0600, 0.071334)),
+	"NTSC 1953":        (2.2,             "C",                     (0.6700, 0.3300, 0.298839), (0.2100, 0.7100, 0.586811), (0.1400, 0.0800, 0.114350)),
+	"PAL/SECAM":        (2.2,             "D65",                   (0.6400, 0.3300, 0.222021), (0.2900, 0.6000, 0.706645), (0.1500, 0.0600, 0.071334)),
 	"ProPhoto RGB":     (1.8,             "D50",                   (0.7347, 0.2653, 0.288040), (0.1596, 0.8404, 0.711874), (0.0366, 0.0001, 0.000086)),
-	"Rec. 709 RGB":     (-709,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
-	"Rec. 2020 RGB":    (-709,            "D65",                   (0.7080, 0.2920, 0.262694), (0.1700, 0.7970, 0.678009), (0.1310, 0.0460, 0.059297)),
-	"SMPTE-C RGB":      (2.2,             "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
-	"SMPTE 240M RGB":   (-240,            "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
+	"Rec. 709":         (-709,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
+	"Rec. 2020":        (-709,            "D65",                   (0.7080, 0.2920, 0.262694), (0.1700, 0.7970, 0.678009), (0.1310, 0.0460, 0.059297)),
+	"Rec. 2020 ST2084": (-2084,           "D65",                   (0.7080, 0.2920, 0.262694), (0.1700, 0.7970, 0.678009), (0.1310, 0.0460, 0.059297)),
+	"SMPTE-C":          (2.2,             "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
+	"SMPTE 240M":       (-240,            "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
 	"sRGB":             (-2.4,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
 	"Wide Gamut RGB":   (2.2,             "D50",                   (0.7350, 0.2650, 0.258187), (0.1150, 0.8260, 0.724938), (0.1570, 0.0180, 0.016875))
 }
@@ -669,6 +646,10 @@ def DIN99b2Lab(L99, a99, b99):
 	return DIN99bcdLCH2Lab(L99, C99, H99, 0, 303.67, .0039, 26, .83, 23, .075)
 
 
+def DIN99bLCH2Lab(L99, C99, H99):
+	return DIN99bcdLCH2Lab(L99, C99, H99, 0, 303.67, .0039, 26, .83, 23, .075)
+
+
 def DIN99c2Lab(L99, a99, b99, whitepoint=None):
 	C99, H99 = DIN99familyab2DIN99CH(a99, b99)
 	return DIN99bcdLCH2Lab(L99, C99, H99, .1, 317.651, .0037, 0, .94, 23, .066,
@@ -681,6 +662,11 @@ def DIN99d2Lab(L99, a99, b99, whitepoint=None):
 						   .06, whitepoint)
 
 
+def DIN99dLCH2Lab(L99, C99, H99, whitepoint=None):
+	return DIN99bcdLCH2Lab(L99, C99, H99, .12, 325.221, .0036, 50, 1.14, 22.5,
+						   .06, whitepoint)
+
+
 def DIN99LCH2Lab(L99, C99, H99, kCH, kE=1.0):
 	G = (math.exp(.045 * C99 * kCH * kE) - 1) / .045
 	return DIN99familyLHCG2Lab(L99, H99, C99, G, kE, 105.51, .0158, 16, .7)
@@ -1047,6 +1033,45 @@ def RGB2HSV(R, G, B, scale=1.0):
 	return H * scale, S * scale, V * scale
 
 
+def RGB2ICtCp(R, G, B, rgb_space="Rec. 2020 ST2084"):
+	# http://www.dolby.com/us/en/technologies/dolby-vision/ICtCp-white-paper.pdf
+	rgb2lms_matrix = Matrix3x3([[1688 / 4096., 2146 / 4096., 262 / 4096.],
+								[683 / 4096., 2951 / 4096., 462 / 4096.],
+								[99 / 4096., 309 / 4096., 3688 / 4096.]])
+	LMS = rgb2lms_matrix * (R, G, B)
+	L_, M_, S_ = (specialpow(FD, 1.0 / -2084) for FD in LMS)
+	L_M_S_2ICtCp_matrix = Matrix3x3([[.5, .5, 0],
+									 [6610 / 4096., -13613 / 4096., 7003 / 4096.],
+									 [17933 / 4096., -17390 / 4096., -543 / 4096.]])
+	I, Ct, Cp = L_M_S_2ICtCp_matrix * (L_, M_, S_)
+	return I, Ct, Cp
+
+
+def ICtCp2RGB(I, Ct, Cp, rgb_space="Rec. 2020 ST2084"):
+	# http://www.dolby.com/us/en/technologies/dolby-vision/ICtCp-white-paper.pdf	
+	L_M_S_2ICtCp_matrix = Matrix3x3([[.5, .5, 0],
+									 [6610 / 4096., -13613 / 4096., 7003 / 4096.],
+									 [17933 / 4096., -17390 / 4096., -543 / 4096.]])
+	L_M_S_ = L_M_S_2ICtCp_matrix.inverted() * (I, Ct, Cp)
+	L, M, S = (specialpow(v, -2084) for v in L_M_S_)
+	rgb2lms_matrix = Matrix3x3([[1688 / 4096., 2146 / 4096., 262 / 4096.],
+								[683 / 4096., 2951 / 4096., 462 / 4096.],
+								[99 / 4096., 309 / 4096., 3688 / 4096.]])
+	R, G, B = rgb2lms_matrix.inverted() * (L, M, S)
+	return R, G, B
+
+
+def XYZ2ICtCp(X, Y, Z, rgb_space="Rec. 2020 ST2084", clamp=False):
+	R, G, B = (specialpow(v, -2084) for v in XYZ2RGB(X, Y, Z, rgb_space,
+													 clamp=clamp))
+	return RGB2ICtCp(R, G, B, rgb_space)
+
+
+def ICtCp2XYZ(I, Ct, Cp, rgb_space="Rec. 2020 ST2084"):
+	R, G, B = (specialpow(v, 1.0 / -2084) for v in ICtCp2RGB(I, Ct, Cp, rgb_space))
+	return RGB2XYZ(R, G, B, rgb_space)
+
+
 def RGB2Lab(R, G, B, rgb_space=None, whitepoint=None, noadapt=False,
 			cat="Bradford"):
 	X, Y, Z = RGB2XYZ(R, G, B, rgb_space, scale=100)
@@ -1114,10 +1139,23 @@ def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
 
 def RGBsaturation(R, G, B, saturation, rgb_space=None):
 	""" (De)saturate a RGB color in CIE xy and return the RGB and xyY values """
-	wx, wy, wY = XYZ2xyY(*RGB2XYZ(1, 1, 1, rgb_space=rgb_space))
-	x, y, Y = XYZ2xyY(*RGB2XYZ(R, G, B, rgb_space=rgb_space))
-	x, y, Y = wx + (x - wx) * saturation,  wy + (y - wy) * saturation, Y
-	return xyY2RGB(x, y, Y, rgb_space=rgb_space), (x, y, Y)
+	whitepoint = RGB2XYZ(1, 1, 1, rgb_space=rgb_space)
+	X, Y, Z = RGB2XYZ(R, G, B, rgb_space=rgb_space)
+	XYZ, xyY = XYZsaturation(X, Y, Z, saturation, whitepoint)
+	return XYZ2RGB(*XYZ, rgb_space=rgb_space), xyY
+
+
+def XYZsaturation(X, Y, Z, saturation, whitepoint=None):
+	""" (De)saturate a XYZ color in CIE xy and return the RGB and xyY values """
+	wx, wy, wY = XYZ2xyY(*get_whitepoint(whitepoint))
+	x, y, Y = XYZ2xyY(X, Y, Z)
+	x, y, Y = xyYsaturation(x, y, Y, wx, wy, saturation)
+	return xyY2XYZ(x, y, Y), (x, y, Y)
+
+
+def xyYsaturation(x, y, Y, wx, wy, saturation):
+	""" (De)saturate a color in CIE xy and return the RGB and xyY values """
+	return wx + (x - wx) * saturation,  wy + (y - wy) * saturation, Y
 
 
 def rgb_to_xyz_matrix(rx, ry, gx, gy, bx, by, whitepoint=None, scale=1.0):
@@ -1159,7 +1197,8 @@ get_rgb_space.cache = {}
 
 
 def get_standard_illuminant(illuminant_name="D50",
-							priority=("ICC", "ASTM E308-01", "Wyszecki & Stiles", None),
+							priority=("ISO 11664-2:2007", "ICC", "ASTM E308-01",
+									  "Wyszecki & Stiles", None),
 							scale=1.0):
 	""" Return a standard illuminant as XYZ coordinates. """
 	cachehash = illuminant_name, tuple(priority), scale
@@ -1404,6 +1443,11 @@ def XYZ2DIN99b(X, Y, Z, whitepoint=None):
 	return Lab2DIN99b(L, a, b)
 
 
+def XYZ2DIN99bLCH(X, Y, Z, whitepoint=None):
+	L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	return Lab2DIN99bLCH(L, a, b)
+
+
 def XYZ2DIN99c(X, Y, Z, whitepoint=None):
 	return XYZ2DIN99cd(X, Y, Z, .1, 317.651, .0037, 0, .94, 23, .066,
 					   whitepoint)
@@ -1427,6 +1471,11 @@ def XYZ2DIN99d(X, Y, Z, whitepoint=None):
 					   whitepoint)
 
 
+def XYZ2DIN99dLCH(X, Y, Z, whitepoint=None):
+	return XYZ2DIN99cdLCH(X, Y, Z, .12, 325.221, .0036, 50, 1.14, 22.5, .06,
+						  whitepoint)
+
+
 def XYZ2Lab(X, Y, Z, whitepoint=None):
 	"""
 	Convert from XYZ to Lab.
@@ -2421,6 +2470,32 @@ cat_matrices = {"Bradford": Matrix3x3([[ 0.89510,  0.26640, -0.16140],
 										  [0, 1, 0],
 										  [0, 0, 1]])}
 
+standard_illuminants = {
+	# 1st level is the standard name => illuminant definitions
+	# 2nd level is the illuminant name => CIE XYZ coordinates
+	# (Y should always assumed to be 1.0 and is not explicitly defined)
+	None: {"E": {"X": 1.00000, "Z": 1.00000}},
+	"ASTM E308-01": {"A": {"X": 1.09850, "Z": 0.35585},
+					 "C": {"X": 0.98074, "Z": 1.18232},
+					 "D50": {"X": 0.96422, "Z": 0.82521},
+					 "D55": {"X": 0.95682, "Z": 0.92149},
+					 "D65": {"X": 0.95047, "Z": 1.08883},
+					 "D75": {"X": 0.94972, "Z": 1.22638},
+					 "F2": {"X": 0.99186, "Z": 0.67393},
+					 "F7": {"X": 0.95041, "Z": 1.08747},
+					 "F11": {"X": 1.00962, "Z": 0.64350}},
+	"ICC": {"D50": {"X": 0.9642, "Z": 0.8249},
+			"D65": {"X": 0.9505, "Z": 1.0890}},
+	"ISO 11664-2:2007": {"D65": {"X": xyY2XYZ(0.3127, 0.329)[0],
+								 "Z": xyY2XYZ(0.3127, 0.329)[2]}},
+	"Wyszecki & Stiles": {"A": {"X": 1.09828, "Z": 0.35547},
+						  "B": {"X": 0.99072, "Z": 0.85223},
+						  "C": {"X": 0.98041, "Z": 1.18103},
+						  "D55": {"X": 0.95642, "Z": 0.92085},
+						  "D65": {"X": 0.95017, "Z": 1.08813},
+						  "D75": {"X": 0.94939, "Z": 1.22558}}
+}
+
 
 def test():
 	for i in range(4):
