@@ -36,6 +36,7 @@ from util_io import StringIOu as StringIO
 from util_os import (expanduseru, expandvarsu, getenvu, is_superuser,
 					 listdir_re, which)
 from util_str import create_replace_function, safe_unicode, strtr
+import colormath
 import encodedstdio
 
 # Runtime configuration
@@ -703,8 +704,8 @@ valid_values = {
 	"synthprofile.trc_gamma_type": ["g", "G"],
 	"tc_algo": ["", "t", "r", "R", "q", "Q", "i", "I"],  # Q = Argyll >= 1.1.0
 	"tc_vrml_use_D50": [0, 1],
-	"tc_vrml_cie_colorspace": ["DIN99", "DIN99b", "DIN99c", "DIN99d", "LCH(ab)",
-							   "LCH(uv)", "Lab", "Luv", "Lu'v'", "xyY"],
+	"tc_vrml_cie_colorspace": ["DIN99", "DIN99b", "DIN99c", "DIN99d", "ICtCp",
+							   "LCH(ab)", "LCH(uv)", "Lab", "Luv", "Lu'v'", "xyY"],
 	"tc_vrml_device_colorspace": ["HSI", "HSL", "HSV", "RGB"],
 	"testchart.auto_optimize": [0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18],
 	"trc": ["240", "709", "l", "s", ""],
@@ -714,12 +715,25 @@ valid_values = {
 	"whitepoint.colortemp.locus": ["t", "T"]
 }
 
+content_rgb_space = colormath.get_rgb_space("DCI P3")
+crx, cry = content_rgb_space[2:][0][:2]
+cgx, cgy = content_rgb_space[2:][1][:2]
+cbx, cby = content_rgb_space[2:][2][:2]
+cwx, cwy = colormath.XYZ2xyY(*content_rgb_space[1])[:2]
 defaults = {
 	"3d.format": "HTML",
 	"3dlut.apply_black_offset": 0,
 	"3dlut.apply_trc": 1,
 	"3dlut.bitdepth.input": 10,
 	"3dlut.bitdepth.output": 12,
+	"3dlut.content.colorspace.blue.x": cbx,
+	"3dlut.content.colorspace.blue.y": cby,
+	"3dlut.content.colorspace.green.x": cgx,
+	"3dlut.content.colorspace.green.y": cgy,
+	"3dlut.content.colorspace.red.x": crx,
+	"3dlut.content.colorspace.red.y": cry,
+	"3dlut.content.colorspace.white.x": cwx,
+	"3dlut.content.colorspace.white.y": cwy,
 	"3dlut.create": 0,
 	"3dlut.trc": "bt1886",
 	"3dlut.trc_gamma": 2.4,
@@ -732,7 +746,8 @@ defaults = {
 	"3dlut.encoding.output.backup": "n",
 	"3dlut.format": "cube",
 	"3dlut.gamap.use_b2a": 0,
-	"3dlut.hdr_peak_luminance": 900,
+	"3dlut.hdr_maxcll": 10000,
+	"3dlut.hdr_peak_luminance": 400,
 	"3dlut.image.layout": "h",
 	"3dlut.image.order": "rgb",
 	"3dlut.input.profile": "",
